<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use Dedi\SyliusQuotePlugin\Api\Command\PickupQuote;
use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Factory\QuoteFactoryInterface;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Doctrine\Persistence\ObjectManager;
use Sylius\Component\Channel\Repository\ChannelRepositoryInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Repository\CustomerRepositoryInterface;
use Sylius\Component\Currency\Model\CurrencyInterface;
use Sylius\Component\Locale\Model\LocaleInterface;
use Sylius\Component\Resource\Generator\RandomnessGeneratorInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class PickupQuoteHandler
{
    public function __construct(
        private readonly QuoteFactoryInterface $quoteFactory,
        private readonly QuoteRepositoryInterface $quoteRepository,
        private readonly ChannelRepositoryInterface $channelRepository,
        private readonly ObjectManager $orderManager,
        private readonly RandomnessGeneratorInterface $generator,
        private readonly CustomerRepositoryInterface $customerRepository,
    ) {
    }

    public function __invoke(PickupQuote $pickupQuote): OrderInterface
    {
        $channel = $this->channelRepository->findOneByCode($pickupQuote->getChannelCode());
        Assert::isInstanceOf($channel, ChannelInterface::class);

        $customer = null;
        if ($pickupQuote->getEmail() !== null) {
            /** @var CustomerInterface|null $customer */
            $customer = $this->customerRepository->findOneBy(['email' => $pickupQuote->getEmail()]);
        }

        if ($customer !== null) {
            /** @var OrderInterface|null $quote */
            $quote = $this->quoteRepository->findLatestNotEmptyQuoteCartByChannelAndCustomer($channel, $customer);
            if ($quote !== null) {
                return $quote;
            }
        }

        /** @var OrderInterface $quote */
        $quote = $this->quoteFactory->createNew();

        /** @var CurrencyInterface $currency */
        $currency = $channel->getBaseCurrency();

        $quote->setChannel($channel);
        $quote->setLocaleCode($this->getLocaleCode($pickupQuote->localeCode, $channel));
        $quote->setCurrencyCode($currency->getCode());
        $quote->setTokenValue($pickupQuote->tokenValue ?? $this->generator->generateUriSafeString(10));

        if ($customer !== null) {
            $quote->setCustomerWithAuthorization($customer);
        }

        $this->orderManager->persist($quote);

        return $quote;
    }

    private function hasLocaleWithLocaleCode(ChannelInterface $channel, ?string $localeCode): bool
    {
        $locales = $channel->getLocales();

        $localeWithExpectedCode = $locales->filter(function (LocaleInterface $locale) use ($localeCode): bool {
            return $locale->getCode() === $localeCode;
        });

        return !$localeWithExpectedCode->isEmpty();
    }

    private function getLocaleCode(?string $localeCode, ChannelInterface $channel): string
    {
        if ($localeCode === null) {
            /** @var LocaleInterface $locale */
            $locale = $channel->getDefaultLocale();

            $localeCode = $locale->getCode();
        }

        if (!$this->hasLocaleWithLocaleCode($channel, $localeCode)) {
            throw new \InvalidArgumentException(sprintf(
                'Cannot pick up cart, locale code "%s" does not exist.',
                $localeCode,
            ));
        }

        return $localeCode;
    }
}
